### ===========================================================================
###
### Part of CPL Restart IPSL packages
###
### ===========================================================================
##
##  Warning, to install, configure, run, use any of Olivier Marti's
##  software or to read the associated documentation you'll need at least
##  one (1) brain in a reasonably working order. Lack of this implement
##  will void any warranties (either express or implied).
##  O. Marti assumes no responsability for errors, omissions,
##  data loss, or any other consequences caused directly or indirectly by
##  the usage of his software by incorrectly or partially configured
##  personal.
##
###
### Documentation : https://forge.ipsl.jussieu.fr/igcmg/wiki/IPSLCM6/MOSAIX
###
## SVN information
#  $Author: omamce $
#  $Date: 2020-09-18 17:02:09 +0200 (Fri, 18 Sep 2020) $
#  $Revision: 5157 $
#  $Id: CreateRestartAtm4Oasis.bash 5157 2020-09-18 15:02:09Z omamce $
#  $HeadURL: svn+ssh://omamce@forge.ipsl.jussieu.fr/ipsl/forge/projets/igcmg/svn/TOOLS/CPLRESTART/CreateRestartAtm4Oasis.bash $
#
'''Create atmospheric restart

## SVN information
Author   = "$Author: omamce $"
Date     = "$Date: 2023-10-25 17:11:15 +0200 (Wed, 25 Oct 2023) $"
Revision = "$Revision: 6666 $"
Id       = "$Id: RunoffWeights.py 6666 2023-10-25 15:11:15Z omamce $"
HeadURL  = "$HeadURL: svn+ssh://omamce@forge.ipsl.jussieu.fr/ipsl/forge/projets/igcmg/svn/TOOLS/MOSAIX/RunoffWeights.py $"
'''

# SVN information
__SVN__ = ({
    'Author'   : '$Author: omamce $',
    'Date'     : '$Date: 2023-10-25 17:11:15 +0200 (Wed, 25 Oct 2023) $',
    'Revision' : '$Revision: 6666 $',
    'Id'       : '$Id: RunoffWeights.py 6666 2023-10-25 15:11:15Z omamce $',
    'HeadURL'  : '$HeadURL: svn+ssh://omamce@forge.ipsl.jussieu.fr/ipsl/forge/projets/igcmg/svn/TOOLS/MOSAIX/RunoffWeights.py $',
    'SVN_Date' : '$SVN_Date: $',
    })
##

import argparse
import textwrap
import time
import os
import sys
import platform
import numpy as np
import xarray as xr

## Reading the command line arguments
#
# Creating a parser
# The first step in using the argparse is creating an ArgumentParser object:
parser = argparse.ArgumentParser (description = textwrap.dedent ("""
   create_flxat
   """), epilog='-------- This is the end of the help message --------')

# Adding arguments
parser.add_argument ('-v', '--verbose', help='verbosity', action="store_true",
                         default=False )
parser.add_argument ('-l', '--level', type=int, default=0,
                         choices=[0, 1, 2], help='verbosity level')
parser.add_argument ('-i', '--input'  , metavar='inputFile'  , nargs='?',
                         default="flxat_fields_notime.nc",
                         help="input file")
parser.add_argument ('-o', '--output' , metavar='outputFile', nargs='?',
                         default="tmp_flxat.nc",
                         help="output file" )
parser.add_argument ('-f', '--format' , metavar='NetCDF_Format', nargs='?',
                         default="NETCDF4",
                         choices=["NETCDF4","NETCDF4_CLASSIC", "NETCDF3_64BIT",
                                   "NETCDF3_CLASSIC", "64bits"],
                         help="NetCDF format")
parser.add_argument ('--IsUnstructured',  choices=['True', 'False'],
                         required=True )

# Parse command line
myargs = parser.parse_args ()

IsUnstructured = myargs.IsUnstructured in [ True, 'True', 'true', 'TRUE' ]
NcFormat = myargs.format
if NcFormat == '64bit' : NcFormat = 'NETCDF4'

## Read Data - promote them to 64 bits
d_In = xr.open_dataset (myargs.input)

lon = d_In.lon.astype(float).values
lat = d_In.lat.astype(float).values

if IsUnstructured :
    dims = lon.shape
else :
    dims = ( lat.shape[0], lon.shape[0] )

# Try to read variables
# Set them to zero if not possible
if 'evap_oce' in d_In.variables  :
    evap_oce  = d_In.evap_oce[0].squeeze().astype(float).values
else                             :
    evap_oce  = np.zeros ( dims )
if 'evap_sic' in d_In.variables  :
    evap_sic  = d_In.evap_sic[0].squeeze().astype(float).values
else :
    evap_sic  = np.zeros ( dims )
if 'fract_oce' in d_In.variables :
    fract_oce = d_In.fract_oce[0].squeeze().astype(float).values
else :
    fract_oce = np.ones ( dims )
if 'fract_sic' in d_In.variables :
    fract_sic = d_In.fract_sic[0].squeeze().astype(float).values
else :
    fract_sic = np.zeros ( dims )
if 'precip' in d_In.variables    :
    precip    = d_In.precip[0].squeeze().astype(float).values
else :
    evap_oce  = np.zeros ( dims )
if 'snow' in d_In.variables      :
    snow      = d_In.snow[0].squeeze().astype(float).values
else :
    snow      = np.zeros ( dims )
if 'soll' in d_In.variables      :
    soll      = d_In.soll[0].squeeze().astype(float).values
else :
    soll      = np.zeros ( dims )
if 'sols' in d_In.variables      :
    sols      = d_In.sols[0].squeeze().astype(float).values
else :
    sols      = np.zeros ( dims )
if 'taux_oce' in d_In.variables  :
    taux_oce  = d_In.taux_oce[0].squeeze().astype(float).values
else :
    taux_oce  = np.zeros ( dims )
if 'taux_sic' in d_In.variables  :
    taux_sic  = d_In.taux_sic[0].squeeze().astype(float).values
else :
    taux_sic  = np.zeros ( dims )
if 'tauy_oce' in d_In.variables  :
    tauy_oce  = d_In.tauy_oce[0].squeeze().astype(float).values
else :
    tauy_oce  = np.zeros ( dims )
if 'tauy_sic' in d_In.variables  :
    tauy_sic  = d_In.tauy_sic[0].squeeze().astype(float).values
else :
    tauy_sic  = np.zeros ( dims )
if 'wind10m' in d_In.variables   :
    wind10m   = d_In.wind10m[0].squeeze().astype(float).values
else :
    wind10m   = np.zeros ( dims )

if IsUnstructured :
    lon       = np.expand_dims ( lon       , axis=1 )
    lat       = np.expand_dims ( lat       , axis=1 )
    evap_oce  = np.expand_dims ( evap_oce  , axis=1 )
    evap_sic  = np.expand_dims ( evap_sic  , axis=1 )
    fract_oce = np.expand_dims ( fract_oce , axis=1 )
    fract_sic = np.expand_dims ( fract_sic , axis=1 )
    precip    = np.expand_dims ( precip    , axis=1 )
    snow      = np.expand_dims ( snow      , axis=1 )
    soll      = np.expand_dims ( soll      , axis=1 )
    sols      = np.expand_dims ( sols      , axis=1 )
    taux_oce  = np.expand_dims ( taux_oce  , axis=1 )
    taux_sic  = np.expand_dims ( taux_sic  , axis=1 )
    tauy_oce  = np.expand_dims ( tauy_oce  , axis=1 )
    tauy_sic  = np.expand_dims ( tauy_sic  , axis=1 )
    wind10m   = np.expand_dims ( wind10m   , axis=1 )
else :
    lon2 = lat [:, np.newaxis]*0 + lon [np.newaxis, :]
    lat2 = lat [:, np.newaxis]   + lon [np.newaxis, :]*0
    lon = lon2
    lat = lat2

##
yxshape = lat.shape
ny, nx = yxshape

## Computations
np.seterr (divide='ignore', invalid='ignore')

## ocean fraction
fract_oce_plus_sic  = fract_oce + fract_sic
# free ocean vs. total ocen fraction
fract_oce_norm = np.where (fract_oce_plus_sic >  0.0,
                               fract_oce/fract_oce_plus_sic, 0.0)
 # sea ice vs. total ocean fraction
fract_sic_norm = np.where (fract_oce_plus_sic >  0.0,
                               fract_sic/fract_oce_plus_sic, 0.0)
##
COTOTRAI = xr.DataArray (precip-snow, dims=('y', 'x'))
COTOTRAI.attrs['long_name'] = 'Liquid precip'
COTOTRAI.attrs['coordinates'] = "lat lon"

COTOTSNO = xr.DataArray (snow       , dims=('y', 'x'))
COTOTSNO.attrs['long_name'] ='Solid precipitation'
COTOTSNO.attrs['coordinates'] = "lat lon"

COTOTEVA = xr.DataArray (evap_oce*fract_oce_norm + evap_sic*fract_sic_norm,
                             dims=('y', 'x'))
COTOTEVA.attrs['coordinates'] = "lat lon"

COICEVAP = xr.DataArray (evap_sic   , dims=('y', 'x'))
COICEVAP.attrs['long_name'] = 'Evaporation on sea ice'
COICEVAP.attrs['coordinates'] = "lat lon"

COQSRMIX = xr.DataArray (sols       , dims=('y', 'x'))
COQSRMIX.attrs['long_name'] = 'Heat flux short wave'
COQSRMIX.attrs['units'] = 'W/m2'
COQSRMIX.attrs['coordinates'] = "lat lon"

COQNSMIX = xr.DataArray (soll       , dims=('y', 'x'))
COQNSMIX.attrs['long_name'] = 'Heat flux minus short wave'
COQNSMIX.attrs['units'] = 'W/m2'
COQNSMIX.attrs['coordinates'] = "lat lon"

COSHFICE = xr.DataArray (sols       , dims=('y', 'x'))
COSHFICE.attrs['long_name'] = 'Heat flux short wave over sea ice'
COSHFICE.attrs['units'] = 'W/m2'
COSHFICE.attrs['coordinates'] = "lat lon"

CONSFICE = xr.DataArray (soll       , dims=('y', 'x'))
CONSFICE.attrs['long_name'] = 'Heat flux minus short wave over sea ice'
CONSFICE.attrs['units'] = 'W/m2'
CONSFICE.attrs['coordinates'] = "lat lon"

COWINDSP = xr.DataArray (wind10m    , dims=('y', 'x'))
COWINDSP.attrs['long_name'] = 'Wind speed at 10m high'
COWINDSP.attrs['units'] = 'm/s'
COWINDSP.attrs['coordinates'] = "lat lon"

CODFLXDT = xr.DataArray (-20.0*np.ones(yxshape), dims=('y', 'x'))
CODFLXDT.attrs['long_name'] = 'dQ/dT - Derivative over temperature of turbulent heat fluxes'
CODFLXDT.attrs['units'] = 'W/m2/K'
CODFLXDT.attrs['coordinates'] = "lat lon"

COCALVIN = xr.DataArray (  0.0*np.ones(yxshape), dims=('y', 'x'))
COCALVIN.attrs['long_name'] = 'Calving of icebergs, solid'
COCALVIN.attrs['coordinates'] = "lat lon"

COLIQRUN = xr.DataArray (  0.0*np.ones(yxshape), dims=('y', 'x'))
COLIQRUN.attrs['long_name'] = 'River run-off, liquid'
COLIQRUN.attrs['coordinates'] = "lat lon"

## Wind stress
tau_x = taux_oce*fract_oce_norm + taux_sic*fract_sic_norm
tau_y = tauy_oce*fract_oce_norm + tauy_sic*fract_sic_norm
COTAUMOD = xr.DataArray (np.sqrt ( (tau_x*tau_x + tau_y*tau_y) ) , dims=('y', 'x'))
COTAUMOD.attrs.update ( {'long_name':'Wind stress modulus', 'units':'Pa',
                             'coordinates':'lat lon' })

## Wind stress, from east/north components to geocentric
rad = np.deg2rad (1.0)
COTAUXXU = xr.DataArray (-tau_x * np.sin(rad * lon)
                        - tau_y * np.sin(rad * lat) * np.cos(rad * lon) ,
                             dims=('y', 'x'))
COTAUYYU = xr.DataArray ( tau_x * np.cos(rad * lon)
                        - tau_y * np.sin(rad * lat) * np.sin(rad * lon) ,
                              dims=('y', 'x'))
COTAUZZU = xr.DataArray ( tau_y * np.cos(rad * lat)                                                 , dims=('y', 'x'))

## Value at North Pole
if IsUnstructured :
    ## Value at North Pole for DYNAMICO grid
    COTAUXXU = xr.where ( lat >= 89.999, -tau_y, COTAUXXU)
    COTAUYYU = xr.where ( lat >= 89.999,  tau_x, COTAUYYU)
    ## Value at South Pole for DYNAMICO grid ?

else :
    ## Value at North Pole for LMDZ lon/lat grid
    COTAUXXU[0,:] = ( -tau_x [0, 0] )
    COTAUYYU[0,:] = ( -tau_y [0, 0] )
    COTAUZZU[0,:] =  0.0
    ## Value at south Pole for LMDZ lon/lat grid
    COTAUXXU[-1,:] = ( -tau_x [-1, 0] )
    COTAUYYU[-1,:] = ( -tau_y [-1, 0] )

##
COTAUXXU.attrs.update ({'long_name':'Wind stress in geocentric referential - x-component',
                            'units':'Pa', 'coordinates':'lat lon', 'Grid':'U' })
COTAUYYU.attrs.update ({'long_name':'Wind stress in geocentric referential - y-component',
                            'units':'Pa', 'coordinates':'lat lon', 'Grid':'U' })
COTAUZZU.attrs.update ({'long_name':'Wind stress in geocentric referential - z-component',
                            'units':'Pa', 'coordinates':'lat lon', 'Grid':'U' })

COTAUXXV = COTAUXXU
COTAUYYV = COTAUYYU
COTAUZZV = COTAUZZU

COTAUXXV.attrs.update ( {'Grid':'V'} )
COTAUYYV.attrs.update ( {'Grid':'V'} )
COTAUZZV.attrs.update ( {'Grid':'V'} )

## check if bounds for lon lat are present and add them to dataset or drop them
ifbnds = 'bounds_lon' in d_In.data_vars and 'bounds_lat' in d_In.data_vars

## Creates final Dataset
lon = xr.DataArray (lon, dims=('y', 'x'))
lon.attrs.update ({'name':'longitude', 'long_name':'Longitude',
                       'units':'degrees_east', 'standard_name':'longitude' })

lat = xr.DataArray (lat, dims=('y', 'x'))
lat.attrs.update ({'name':'latitude', 'long_name':'Latitude',
                       'units':'degrees_north', 'standard_name':'latitude' })

if ifbnds :
    lon.attrs['bounds'] = 'bounds_lon'
    lat.attrs['bounds'] = 'bounds_lat'

    bounds_lon = d_In.bounds_lon.values.astype (float)
    bounds_lat = d_In.bounds_lat.values.astype (float)
    nvertex = bounds_lon.shape[-1]

    bounds_lon = xr.DataArray ( np.reshape (bounds_lon, (ny, nx, nvertex)),
                                    dims=('y', 'x', 'nvertex'))
    bounds_lat = xr.DataArray ( np.reshape (bounds_lat, (ny, nx, nvertex)),
                                    dims=('y', 'x', 'nvertex'))

    bounds_lon.attrs['units'] = 'degrees_east'
    bounds_lat.attrs['units'] = 'degrees_north'

# Prepares dictionnary to export dataset to netcdf file with or without bounds
dictdataset = {'lat':lat, 'lon':lon }
if ifbnds:
    dictdataset.update ( {'bounds_lon':bounds_lon,
                          'bounds_lat':bounds_lat} )
dictdataset.update ( {
    'COTOTRAI':COTOTRAI, 'COTOTSNO':COTOTSNO, 'COTOTEVA':COTOTEVA,
    'COICEVAP':COICEVAP, 'COQSRMIX':COQSRMIX, 'COQNSMIX':COQNSMIX,
    'COSHFICE':COSHFICE, 'CONSFICE':CONSFICE, 'CODFLXDT':CODFLXDT,
    'COCALVIN':COCALVIN, 'COLIQRUN':COLIQRUN, 'COWINDSP':COWINDSP,
    'COTAUMOD':COTAUMOD, 'COTAUXXU':COTAUXXU, 'COTAUYYU':COTAUYYU,
    'COTAUZZU':COTAUZZU, 'COTAUXXV':COTAUXXV, 'COTAUYYV':COTAUYYV,
    'COTAUZZV':COTAUZZV } )

d_Out = xr.Dataset (dictdataset)

d_Out.attrs.update ( {
    'AssociatedFiles' : myargs.input,
    'Conventions'     : "CF-1.6",
    'source'          : "IPSL Earth system model",
    'group'           : "ICMC IPSL Climate Modelling Center",
    'Institution'     : "IPSL https://www.ipsl.fr",
    'Model'           : "IPSL CM",
    'description'     : "Fields needed by OASIS-MCT",
    'Program'         : f'Generated by {sys.argv[0]} with flags ' + ' '.join (sys.argv[1:]),
    'timeStamp'       : time.asctime (),
    'directory'       : os.getcwd (),
    'HOSTNAME'        : platform.node (),
    'LOGNAME'         : os.getlogin (),
    'Python'          : "Python version " +  platform.python_version (),
    'OS'              : platform.system (),
    'release'         : platform.release (),
    'hardware'        : platform.machine (),
    'SVN_Author'      : "$Author: omamce $",
    'SVN_Date'        : "$Date: 2022-07-06 11:06:07 +0200 (Wed, 06 Jul 2022) $",
    'SVN_Revision'    : "$Revision: 6190 $",
    'SVN_Id'          : "$Id: CalvingWeights.py 6190 2022-07-06 09:06:07Z omamce $",
    'SVN_HeadURL'     : "$HeadURL: svn+ssh://omamce@forge.ipsl.jussieu.fr/ipsl/forge/projets/igcmg/svn/TOOLS/MOSAIX/CalvingWeights.py $",
    } )

d_Out.to_netcdf ( myargs.output, format=NcFormat )
d_Out.close ()
## ===========================================================================
##
##                               That's all folk's !!!
##
## ===========================================================================
